/*
 * MiniPC board late initialisation, based on uboot-xburst and xburst-tools.
 *
 * Copyright (C) 2000-2009 Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 * Copyright (C) 2005-2006 Ingenic Semiconductor, <jlwei@ingenic.cn>
 * Copyright (C) Xiangfu Liu <xiangfu.z@gmail.com>
 * Copyright (C) 2015, 2016, 2017 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board.h"
#include "minipc.h"
#include "gpio.h"

/* Later initialisation functions. */

void gpio_init2()
{
	void *gpio_port_base;

	/* LED enable. */

	gpio_port_base = jz4740_get_port((void *) GPIO_BASE, GPIO_PORT_LED);
        jz4740_gpio_as_output(gpio_port_base, GPIO_LED_EN);
        jz4740_gpio_set_pin(gpio_port_base, GPIO_LED_EN);

	__harb_usb0_uhc();

	/* gpio_as_emc */

	gpio_port_base = jz4740_get_port((void *) GPIO_BASE, GPIO_PORT_EMC);

	jz4740_gpio_ctrl_update(gpio_port_base, GPIO_GPALR, 0xC0000000, 0x40000000);
	jz4740_gpio_ctrl_update(gpio_port_base, GPIO_GPAUR, 0x0000FFFF, 0x00005555);

	/* gpio_as_dma */

	gpio_port_base = jz4740_get_port((void *) GPIO_BASE, GPIO_PORT_DMA);

	jz4740_gpio_ctrl_update(gpio_port_base, GPIO_GPALR, 0xFF000000, 0x55000000);
	jz4740_gpio_ctrl_update(gpio_port_base, GPIO_GPAUR, 0x00F00000, 0x00500000);

	/* Initialise LCD pins. */

	/* gpio_as_lcd_master */

	gpio_port_base = jz4740_get_port((void *) GPIO_BASE, GPIO_PORT_LCD);

	jz4740_gpio_ctrl_update(gpio_port_base, GPIO_GPALR, 0xFFFF0000, 0x55550000);
	jz4740_gpio_ctrl_set(gpio_port_base, GPIO_GPAUR, 0x556A5555);

	/* Initialise MSC pins. */

	/* gpio_as_msc */

	gpio_port_base = jz4740_get_port((void *) GPIO_BASE, GPIO_PORT_MSC);

	jz4740_gpio_ctrl_update(gpio_port_base, GPIO_GPALR, 0x0000FFF0, 0x00005550);
}

void rtc_init()
{
	/* NOTE: May only be accessible via I2C. */
}

/* Timer routines. */

unsigned long timestamp;
unsigned long lastdec;

/*
 * Timer without interrupts.
 */

int timer_init()
{
        __ost_set_clock(TIMER_CHAN, OST_TCSR_CKS_PCLK_256);
        __ost_set_reload(TIMER_CHAN, TIMER_FDATA);
        __ost_set_count(TIMER_CHAN, TIMER_FDATA);
        __ost_enable_channel(TIMER_CHAN);

	jz4740_cpm_start_clock((void *) CPM_BASE);

        lastdec = TIMER_FDATA;
        timestamp = 0;

        return 0;
}

/* Timer interrupt activation. */

void timer_init_irq()
{
	__ost_enable_interrupt(TIMER_CHAN);
	/* NOTE: Need flag clearing? */
	__intc_unmask_irq(TIMER_CHAN_IRQ);
}

void timer_clear()
{
	__intc_ack_irq(TIMER_CHAN_IRQ);
	__ost_clear_uf(TIMER_CHAN);
}

/* GPIO interrupt activation. */

void gpio_init_irq()
{
	gpio_port_base = jz4740_get_port((void *) GPIO_BASE, GPIO_PORT_POWER);
	jz4740_gpio_as_irq(gpio_port_base, GPIO_POWER, gpio_irq_low_level);
	__intc_unmask_irq(GPIO_IRQ);
}

int gpio_have_irq(uint8_t gpio)
{
	gpio_port_base = jz4740_get_port((void *) GPIO_BASE, gpio / 32);
	return jz4740_gpio_have_irq(gpio_port_base, gpio % 32);
}

/* Board startup detection. */

int is_started()
{
	return jz4740_cpm_have_clock((void *) CPM_BASE);
}
